"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
class Endpoint {
    constructor(endpointInfo, controllerFactory) {
        this.endpointInfo = endpointInfo;
        this.controllerFactory = controllerFactory;
    }
    register(api) {
        let registerMethod = this.mapHttpMethodToCall(api, this.endpointInfo.httpMethod);
        let rootPath = this.endpointInfo.controller.path || "";
        let endpointPath = this.endpointInfo.path || "";
        let path = `${rootPath}${endpointPath}`;
        registerMethod(path, async (req, res) => await this.invoke(this, req, res));
    }
    // entry point or lambda-api request engine, self parameter required to inject
    // this instance into the invocation context as `this` is overwritten
    async invoke(self, request, response) {
        // build a instance of the associated controller
        let controller = self.controllerFactory(self.endpointInfo.controller.classConstructor);
        let produces;
        if (self.endpointInfo.produces) {
            produces = self.endpointInfo.produces;
        }
        else if (self.endpointInfo.controller.produces) {
            produces = self.endpointInfo.controller.produces;
        }
        if (produces) {
            response.removeHeader("Content-Type")
                .header("Content-Type", produces);
        }
        if (typeof (controller.setRequest) === "function") {
            controller.setRequest(request);
        }
        if (typeof (controller.setResponse) === "function") {
            controller.setResponse(response);
        }
        let endpointResponse = await this.invokeControllerMethod(controller, self.endpointInfo.methodName, request, response);
        let rawRes = response;
        if (!endpointResponse && rawRes && rawRes._state !== "done") {
            throw "no content was set in response or returned by endpoint method, " +
                `path: ${self.endpointInfo.path} | endpoint: ${self.endpointInfo.name}`;
        }
        return endpointResponse;
    }
    mapHttpMethodToCall(api, method) {
        if (method == "GET") {
            return api.get;
        }
        else if (method == "POST") {
            return api.post;
        }
        else if (method == "PUT") {
            return api.put;
        }
        else if (method == "PATCH") {
            return api.patch;
        }
        else if (method == "DELETE") {
            return api.delete;
        }
        throw `Unrecognised HTTP method ${method}`;
    }
    async invokeControllerMethod(controller, methodName, request, response) {
        var method = controller[methodName];
        var parameters = this.buildEndpointParameters(request, response);
        return await method.apply(controller, parameters);
    }
    buildEndpointParameters(request, response) {
        return this.endpointInfo
            .parameterExtractors
            .map(pe => pe.extract(request, response));
    }
}
exports.Endpoint = Endpoint;
//# sourceMappingURL=Endpoint.js.map