"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const ApiRequest_1 = require("../model/ApiRequest");
class RequestBuilder {
    constructor(method, path) {
        this.method = method;
        this.path = path;
        this.httpHeaders = {};
        this.httpQueryParams = {};
    }
    static get(path) {
        return new RequestBuilder("GET", path);
    }
    static post(path) {
        return new RequestBuilder("POST", path);
    }
    static put(path) {
        return new RequestBuilder("PUT", path);
    }
    static patch(path) {
        return new RequestBuilder("PATCH", path);
    }
    static delete(path) {
        return new RequestBuilder("DELETE", path);
    }
    static do(method, path) {
        return new RequestBuilder(method, path);
    }
    header(key, value) {
        this.httpHeaders[key] = value;
        return this;
    }
    headers(httpHeaders) {
        for (let key in httpHeaders) {
            this.httpHeaders[key] = httpHeaders[key];
        }
        return this;
    }
    query(key, value) {
        this.httpQueryParams[key] = value;
        return this;
    }
    queryParams(params) {
        for (let key in params) {
            this.httpQueryParams[key] = params[key];
        }
        return this;
    }
    body(value) {
        this.httpBody = value;
        return this;
    }
    mapToObject(map) {
        let obj = {};
        for (let key in map) {
            obj[key] = map[key];
        }
        return obj;
    }
    build() {
        let request = new ApiRequest_1.ApiRequest();
        request.path = this.path;
        request.httpMethod = this.method;
        request.headers = this.mapToObject(this.httpHeaders);
        request.queryStringParameters = this.mapToObject(this.httpQueryParams);
        request.body = this.httpBody;
        request.isBase64Encoded = false;
        return request;
    }
}
exports.RequestBuilder = RequestBuilder;
//# sourceMappingURL=RequestBuilder.js.map